/******************************************************************************
 * Includes
 *****************************************************************************/
#include "lpc2xxx.h"

#include "hw.h"
#include "pins.h"
#if (HWVER != 0)
#include "eeprom.h"
#endif

/******************************************************************************
 * Typedefs and defines
 *****************************************************************************/

/*****************************************************************************
 * Global variables
 ****************************************************************************/

/*****************************************************************************
 * Local variables
 ****************************************************************************/
#if (HWVER != 0)
static unsigned char greenLedShadow;
static unsigned char btResetShadow;
#endif

/*****************************************************************************
 * Local prototypes
 ****************************************************************************/


/*****************************************************************************
 *
 * Description:
 *    Initialize the io-pins and find out if HW is ver 1.0 or ver 1.1
 *
 ****************************************************************************/
void immediateIoInit(void) {
  //make all key signals as inputs
  IODIR &= ~(KEYPIN_CENTER | KEYPIN_UP | KEYPIN_DOWN | KEYPIN_LEFT | KEYPIN_RIGHT);

  IODIR |= BUZZER_PIN;
  IOSET  = BUZZER_PIN;

  IODIR |= BACKLIGHT_PIN;
  IOSET  = BACKLIGHT_PIN;

#if (HWVER == 0)
  IODIR |= LCD_RST;
  IOCLR  = LCD_RST;
  
  IODIR |= BT_RST;
  IOCLR  = BT_RST;
  
  IODIR |= (LED_GREEN_PIN | LED_RED_PIN);
  IOSET  = (LED_GREEN_PIN | LED_RED_PIN);
#else
  {
  unsigned char initCommand[] = {0x12, 0x97, 0x80, 0x00, 0x40, 0x00, 0x14, 0x00, 0x00};
  //                                                                   04 = LCD_RST# low
  //                                                                   10 = BT_RST# low

  //initialize PCA9532
  i2cInit();

  if (I2C_CODE_OK == pca9532(initCommand, sizeof(initCommand), NULL, 0)) {
    greenLedShadow = FALSE;
    btResetShadow  = TRUE;
  }
  }
#endif
}


/*****************************************************************************
 *
 * Description:
 *    Reset the LCD (by strobing the LCD reset pin)
 *
 ****************************************************************************/
void resetLCD(void) {
#if (HWVER == 0)
  IOCLR  = LCD_RST;
  sleep(2);
  IOSET  = LCD_RST;
  sleep(5);
#else
  unsigned char command[] = {0x07, 0x00};
  //                                 04 = LCD_RST# low
  //                                 00 = LCD_RST# high
  command[1] = 0x04;
  pca9532(command, sizeof(command), NULL, 0);
  sleep(2);
  command[1] = 0x00;
  pca9532(command, sizeof(command), NULL, 0);
  sleep(5);
#endif
}


/*****************************************************************************
 *
 * Description:
 *    Controls the reset signal to the BGB203-S06 Bluetooth moduls
 *
 ****************************************************************************/
void resetBT(unsigned char resetFlag) {
#if (HWVER == 0)
  if (TRUE == resetFlag)
    IOCLR  = BT_RST;
  else
    IOSET  = BT_RST;
#else
  unsigned char command[] = {0x07, 0x00};
  //                                 10 = BT_RST# low
  //                                 00 = BT_RST# high
  //                                 40 = Green LED on  (must handle this also)
  btResetShadow = resetFlag;
  if (TRUE == resetFlag)
    command[1] = 0x10;
  if (TRUE == greenLedShadow)
    command[1] |= 0x40;
  pca9532(command, sizeof(command), NULL, 0);
#endif
}


/*****************************************************************************
 *
 * Description:
 *    Controls the buzzer (on or off)
 *
 ****************************************************************************/
void setBuzzer(unsigned char on) {
  if (TRUE == on)
    IOCLR = BUZZER_PIN;
  else
    IOSET = BUZZER_PIN;
}


/*****************************************************************************
 *
 * Description:
 *    Controls the two LEDs
 *
 ****************************************************************************/
void setLED(unsigned char ledSelect, unsigned char ledState) {
#if (HWVER == 0)
  if (LED_GREEN == ledSelect) {
    if (TRUE == ledState)
      IOCLR = LED_GREEN_PIN;
    else
      IOSET = LED_GREEN_PIN;
  } else if (LED_RED == ledSelect) {
    if (TRUE == ledState)
      IOCLR = LED_RED_PIN;
    else
      IOSET = LED_RED_PIN;
  }
#else
  unsigned char commandLedGreen[] = {0x07, 0x00};
  //                                         40 = LED on
  //                                         00 = LED off
  //                                         10 = BT_RST# low   (must handle this also)
  unsigned char commandLedRed[] =   {0x08, 0x00};
  //                                         01 = LED on
  //                                         00 = LED off
  if (LED_GREEN == ledSelect) {
    greenLedShadow = ledState;
    if (TRUE == ledState)
      commandLedGreen[1] = 0x40;
    if(TRUE == btResetShadow)
      commandLedGreen[1] |= 0x10;
    pca9532(commandLedGreen, sizeof(commandLedGreen), NULL, 0);
  } else if (LED_RED == ledSelect) {
    if (TRUE == ledState)
      commandLedRed[1] = 0x01;
    pca9532(commandLedRed, sizeof(commandLedRed), NULL, 0);
  }
#endif
}


/*****************************************************************************
 *
 * Description:
 *    Get current state of joystick switch
 *
 ****************************************************************************/
unsigned char getKeys(void) {
  unsigned char readKeys = KEY_NOTHING;

#if (HWVER == 0)
  if ((IOPIN & KEYPIN_CENTER) == 0) readKeys |= KEY_CENTER;
  if ((IOPIN & KEYPIN_UP) == 0)     readKeys |= KEY_UP;
  if ((IOPIN & KEYPIN_DOWN) == 0)   readKeys |= KEY_DOWN;
  if ((IOPIN & KEYPIN_LEFT) == 0)   readKeys |= KEY_LEFT;
  if ((IOPIN & KEYPIN_RIGHT) == 0)  readKeys |= KEY_RIGHT;
#else
  unsigned char commandReadKeys[] = {0x00};
  unsigned char keySample;
  pca9532(commandReadKeys, sizeof(commandReadKeys), &keySample, 1);
  if ((keySample & 0x01) == 0) readKeys |= KEY_CENTER;
  if ((keySample & 0x04) == 0) readKeys |= KEY_UP;
  if ((keySample & 0x10) == 0) readKeys |= KEY_DOWN;
  if ((keySample & 0x02) == 0) readKeys |= KEY_LEFT;
  if ((keySample & 0x08) == 0) readKeys |= KEY_RIGHT;
#endif

  return readKeys;
}


/*****************************************************************************
 *
 * Description:
 *    Select/deselect LCD controller (by controlling chip select signal)
 *
 ****************************************************************************/
void selectLCD(unsigned char select) {
#if (HWVER == 0)
  if (TRUE == select)
    IOCLR = LCD_CS_V1_0;
  else
    IOSET = LCD_CS_V1_0;
#else
  if (TRUE == select)
    IOCLR = LCD_CS_V1_1;
  else
    IOSET = LCD_CS_V1_1;
#endif
}


/*****************************************************************************
 *
 * Description:
 *    Send 9-bit data to LCD controller
 *
 ****************************************************************************/
void sendToLCD(unsigned char firstBit, unsigned char data) {
  //disable SPI
  IOCLR = LCD_CLK;
  PINSEL0 &= 0xffff00ff;
  
  if (1 == firstBit)
    IOSET = LCD_MOSI;   //set MOSI
  else
    IOCLR = LCD_MOSI;   //reset MOSI
  
  //Set clock high
  IOSET = LCD_CLK;
  
  //Set clock low
  IOCLR = LCD_CLK;
  
  /*
   * Enable SPI again
   */
  //initialize SPI interface
  SPI_SPCCR = 0x08;    
  SPI_SPCR  = 0x20;

  //connect SPI bus to IO-pins
  PINSEL0 |= 0x00005500;
  
  //send byte
  SPI_SPDR = data;
  while((SPI_SPSR & 0x80) == 0)
    ;
}


/*****************************************************************************
 *
 * Description:
 *    Initialize the SPI interface for the LCD controller
 *
 ****************************************************************************/
void initSpiForLcd(void) {
  //make SPI slave chip select an output and set signal high
#if (HWVER == 0)
  IODIR |= (LCD_CS_V1_0 | LCD_CLK | LCD_MOSI);
#else
  IODIR |= (LCD_CS_V1_1 | LCD_CLK | LCD_MOSI);
#endif
  
  //deselect controller
  selectLCD(FALSE);

  //connect SPI bus to IO-pins
  PINSEL0 |= 0x00005500;
  
  //initialize SPI interface
  SPI_SPCCR = 0x08;    
  SPI_SPCR  = 0x20;
}

void sleep(unsigned long ticks) {
  unsigned long future = CTC + ticks;
  while(future > CTC) {
    ;
  }
}
