/******************************************************************************
 *
  *
 * File:
 *    plasma.c
 *
 * Description:
 *    Main function of the "LPC2104 Color LCD Game Board with Bluetooth"
 *
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/
#include "ea_init.h"
#include <stdlib.h>
#include "hw.h"

/******************************************************************************
 * Typedefs and defines
 *****************************************************************************/
#define PROC1_STACK_SIZE 800
#define INIT_STACK_SIZE  600

#define LCD_CMD_SWRESET   0x01
#define LCD_CMD_BSTRON    0x03
#define LCD_CMD_SLEEPIN   0x10
#define LCD_CMD_SLEEPOUT  0x11
#define LCD_CMD_INVON     0x21
#define LCD_CMD_SETCON    0x25
#define LCD_CMD_DISPON    0x29
#define LCD_CMD_CASET     0x2A
#define LCD_CMD_PASET     0x2B
#define LCD_CMD_RAMWR     0x2C
#define LCD_CMD_RGBSET    0x2D
#define LCD_CMD_MADCTL    0x36
#define LCD_CMD_COLMOD    0x3A

#define MADCTL_HORIZ      0x48
#define MADCTL_VERT       0x68

#define N_COLORS         192
#define RGB2SHORT(r,g,b) (((r) & 0x0F) << 8) | (((g) & 0x0F) << 4) | (((b) & 0x0F))
/*****************************************************************************
 * Global variables
 ****************************************************************************/
volatile unsigned long ms;


/*****************************************************************************
 * Local variables
 ****************************************************************************/
static unsigned char costbl[] = {64, 64, 64, 64, 64, 64, 64, 64, 63, 63, 63, 63, 63, 62, 62, 62,
                                 62, 61, 61, 61, 60, 60, 59, 59, 59, 58, 58, 57, 57, 56, 56, 55,
                                 55, 54, 53, 53, 52, 52, 51, 50, 50, 49, 48, 48, 47, 46, 46, 45,
                                 44, 44, 43, 42, 41, 41, 40, 39, 38, 37, 37, 36, 35, 34, 34, 33,
                                 32, 31, 30, 30, 29, 28, 27, 27, 26, 25, 24, 23, 23, 22, 21, 20,
                                 20, 19, 18, 18, 17, 16, 16, 15, 14, 14, 13, 12, 12, 11, 11, 10,
                                  9,  9,  8,  8,  7,  7,  6,  6,  5,  5,  5,  4,  4,  3,  3,  3,
                                  2,  2,  2,  2,  1,  1,  1,  1,  1,  0,  0,  0,  0,  0,  0,  0,
                                  0,  0,  0,  0,  0,  0,  0,  0,  1,  1,  1,  1,  1,  2,  2,  2,
                                  2,  3,  3,  3,  4,  4,  5,  5,  5,  6,  6,  7,  7,  8,  8,  9,
                                  9, 10, 11, 11, 12, 12, 13, 14, 14, 15, 16, 16, 17, 18, 18, 19,
                                 20, 20, 21, 22, 23, 23, 24, 25, 26, 27, 27, 28, 29, 30, 30, 31,
                                 32, 33, 34, 34, 35, 36, 37, 37, 38, 39, 40, 41, 41, 42, 43, 44,
                                 44, 45, 46, 46, 47, 48, 48, 49, 50, 50, 51, 52, 52, 53, 53, 54,
                                 55, 55, 56, 56, 57, 57, 58, 58, 59, 59, 59, 60, 60, 61, 61, 61,
                                 62, 62, 62, 62, 63, 63, 63, 63, 63, 64, 64, 64, 64, 64, 64, 64};
static unsigned short lut[256];

/*****************************************************************************
 * Local prototypes
 ****************************************************************************/
void genLUT(void);
void drawPlasma(const unsigned char *p);
void movePlasma(unsigned char *p);

void lcdWrcmd(unsigned char data) { sendToLCD(0, data); }
void lcdWrdata(unsigned char data) { sendToLCD(1, data); }

void lcdContrast(unsigned char cont) //vary between 0 - 127
{
  //select controller
  selectLCD(TRUE);

  //set contrast cmd.
  lcdWrcmd(LCD_CMD_SETCON);
  lcdWrdata(cont & 0x7F);

  //deselect controller
  selectLCD(FALSE);
}

void lcdInit(void) {
  int i;

  //init SPI interface
  initSpiForLcd();
  
  //select controller
  selectLCD(TRUE);

  lcdWrcmd(LCD_CMD_SWRESET);

  sleep(1);
  lcdWrcmd(LCD_CMD_SLEEPOUT);
  lcdWrcmd(LCD_CMD_DISPON);
  lcdWrcmd(LCD_CMD_BSTRON);
  sleep(1);
		
  lcdWrcmd(LCD_CMD_MADCTL);   //Memory data acces control
  lcdWrdata(MADCTL_HORIZ);    //X Mirror and BGR format
  lcdWrcmd(LCD_CMD_COLMOD);   //Colour mode
  lcdWrdata(0x06);            //4K colour mode Type B select
  lcdWrcmd(LCD_CMD_INVON);    //Non Invert mode

  lcdWrcmd(LCD_CMD_RGBSET);   //LUT write
  lcdWrdata(0);               //Red
  lcdWrdata(2);
  lcdWrdata(4);
  lcdWrdata(6);
  lcdWrdata(9);
  lcdWrdata(11);
  lcdWrdata(13);
  lcdWrdata(15);
  lcdWrdata(0);               //Green
  lcdWrdata(2);
  lcdWrdata(4);
  lcdWrdata(6);
  lcdWrdata(9);
  lcdWrdata(11);
  lcdWrdata(13);
  lcdWrdata(15);
  lcdWrdata(0);               //Blue
  lcdWrdata(6);
  lcdWrdata(10);
  lcdWrdata(15);

  lcdWrcmd(LCD_CMD_CASET);    //set X
  lcdWrdata(0);
  lcdWrdata(129);
  lcdWrcmd(LCD_CMD_PASET);    //set Y
  lcdWrdata(0);
  lcdWrdata(129);
  lcdWrcmd(LCD_CMD_RAMWR);    //write memory
  for(i=0; i<33800; i++)
    lcdWrdata(0);

  //deselect controller
  selectLCD(FALSE);

  lcdContrast(56);
}


/*****************************************************************************
 *
 * Description:
 *    The first function to execute 
 *
 ****************************************************************************/
int main(void) {
  unsigned char p[5];

  //immediate initilaizeation of hardware I/O pins
  immediateIoInit();
  
  resetLCD();
  lcdInit();

  genLUT();

  while (1) {
    drawPlasma(p);
    movePlasma(p);
  }

  return 0;
}



/*****************************************************************************
 *
 * Description:
 *    The timer tick entry function that is called once every timer tick
 *    interrupt in the RTOS. Observe that any processing in this
 *    function must be kept as short as possible since this function
 *    execute in interrupt context.
 *
 * Params:
 *    [in] elapsedTime - The number of elapsed milliseconds since last call.
 *
 ****************************************************************************/
void appTick(unsigned long elapsedTime) {
  ms += elapsedTime;
}

void genLUT(void) {
  int i;
  /* Orginal 
  for (i=0; i<64; i++) {
    lut[    i] = RGB2SHORT(      15,      i/4, (63-i)/4);
    lut[ 64+i] = RGB2SHORT((63-i)/4,       15,      i/4);
    lut[128+i] = RGB2SHORT(       0, (63-i)/4,       15);
    lut[192+i] = RGB2SHORT(     i/4,        0,       15);
  }
  */
  for (i=0; i<16; i++) {
    lut[    2*i  ] = RGB2SHORT(  15,    i,    0);
    lut[    2*i+1] = RGB2SHORT(  15,    i,    0);
    lut[ 32+2*i  ] = RGB2SHORT(15-i,   15,    0);
    lut[ 32+2*i+1] = RGB2SHORT(15-i,   15,    0);
    lut[ 64+2*i  ] = RGB2SHORT(   0,   15,    i);
    lut[ 64+2*i+1] = RGB2SHORT(   0,   15,    i);
    lut[ 96+2*i  ] = RGB2SHORT(   0, 15-i,   15);
    lut[ 96+2*i+1] = RGB2SHORT(   0, 15-i,   15);
    lut[128+2*i  ] = RGB2SHORT(   i,    0,   15);
    lut[128+2*i+1] = RGB2SHORT(   i,    0,   15);
    lut[160+2*i  ] = RGB2SHORT(  15,    0, 15-i);
    lut[160+2*i+1] = RGB2SHORT(  15,    0, 15-i);
  }
}

void drawPlasma(const unsigned char *p) {
  unsigned int i,j;
  unsigned int t0, t1, t2, t3;
  unsigned int ci, cj;
  unsigned short rgb;

  selectLCD(TRUE);
  lcdWrcmd(LCD_CMD_CASET);        //set X
  lcdWrdata(0);
  lcdWrdata(129);

  lcdWrcmd(LCD_CMD_PASET);        //set Y
  lcdWrdata(0);
  lcdWrdata(129);

  lcdWrcmd(LCD_CMD_RAMWR);        //write memory

  t2 = p[2];
  t3 = p[3];
  for (i=0; i<=129; i++) {
    ci = p[5] + costbl[t2 & 0xFF] + costbl[t3 & 0xFF] + costbl[i & 0xFF];
    t0 = p[0];
    t1 = p[1];
    for (j=0; j<=129; j++) {
      cj = ci + costbl[t0 & 0xFF] + costbl[t1 & 0xFF] + costbl[j & 0xFF];
      rgb = lut[cj % N_COLORS];
      lcdWrdata((unsigned char) (rgb >> 8));
      lcdWrdata((unsigned char) (rgb & 0xFF));
      t0 += 3;
      t1 += 1;
    }
    t2 += 2;
    t3 += 3;
  }
  selectLCD(FALSE);
}

void movePlasma(unsigned char *p) {
  p[0] += rand() % 4 + 4;
  p[1] -= rand() % 5 + 3;
  p[2] += rand() % 6 + 2;
  p[3] -= rand() % 7 + 1;
  p[5] =(p[5]+1) % N_COLORS;
}
