/******************************************************************************
 *
 *
 * File:
 *    styx.c
 *
 * Description:
 *    Implements the styx game.
 *
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/
#include "../pre_emptive_os/api/osapi.h"
#include "../pre_emptive_os/api/general.h"
//#include <printf_P.h>
#include <ea_init.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <limits.h>
#include "lcd.h"
#include "hw.h"
#include "key.h"
#include "select.h"
//#include "picture.h"

/******************************************************************************
 * Typedefs and defines
 *****************************************************************************/
#define BOARD_XMIN 10
#define BOARD_XMAX 118
#define BOARD_YMIN 20
#define BOARD_YMAX 108

#define POLY_LENGTH 200

#define LINE_COLOR 0xFF
#define BG_COLOR 0x00

#define MAX_LEVEL 6

#define CURSOR_STEP 2
#define PAUSE_LENGTH 2

#define SELF_INTERSECT 1
#define PATH_MADE 2

typedef tU8 uint8_t;

typedef struct {
    int x, y;
} point_t;

/* Important! keep closed polygons counter-clockwise */
typedef struct {
    int n;
    point_t p[POLY_LENGTH];
} poly_t;

#define STYX_LENGTH 10
#define STYX_MAX_STREACH 200
#define STYX_COLOR_BASE 1
#define STYX_COLOR_CYCLE 128

typedef struct {
    point_t p0, p1;
} line_t;

typedef struct {
    line_t delta, lines[STYX_LENGTH];
    int current;
    uint8_t color;
} styx_t;

typedef struct CURSOR {
    point_t p;
    uint8_t direction;
    int state;
} cursor_t;

typedef struct IMAGE {
    uint8_t width, height, compression, escapechar;
    uint8_t data[130][130];
} image_t;

#define LCD_CMD_CASET     0x2A
#define LCD_CMD_PASET     0x2B
#define LCD_CMD_RAMWR     0x2C

/*****************************************************************************
 * Local prototypes
 ****************************************************************************/
void init_level(void);
void init_cursor(cursor_t * c);
void init_styx(styx_t * s);
int advance_cursor(const uint8_t k, cursor_t * c);
int advance_styx(styx_t * s);
void showScore(void);
void draw_ortho(const point_t * p0, const point_t * p1, const uint8_t color);
void clip_line(line_t * l);
void draw_line(const line_t * l, const uint8_t color);
void put_pixel(const int x, const int y, const uint8_t color);
int inside(const point_t * pt, const poly_t * py);
int check_point(const point_t * pt, const poly_t * py);
int check_line(const line_t * l, const poly_t * p);
void partition(const poly_t * b, const poly_t * p, poly_t * r1, poly_t * r2);
int fill_poly(const poly_t * p, const int level);
int fill_line(const int y, const int x0, const int x1, const int level);

/*****************************************************************************
 * Local variables
 ****************************************************************************/
unsigned int score;

poly_t regions[4];
poly_t *border, *path, *region1, *region2;

#include "earth.c"
#include "liftoff.c"
#include "mercury.c"
#include "iss.c"
#include "satelite.c"
#include "lander.c"
const unsigned char *const levelmap[MAX_LEVEL] = {_earth, _liftoff, _mercury, 
                                                  _iss, _satelite, _lander};

static int high_score = 0;

/*****************************************************************************
 * External variables
 ****************************************************************************/
extern volatile tU32 ms;


/*****************************************************************************
 *
 * Description:
 *    Implement Styx game
 *
 ****************************************************************************/
void playStyx(void)
{
    cursor_t cur;
    styx_t styx;
    int speed, level, level_score;
    int game_over, event, done;
    uint8_t keypress;

    int size;

    //game loop
    do {
        level = 0;
        speed = 14;
        score = 0;
        level_score = 0;
        game_over = FALSE;
        srand(ms);              //Ensure random seed initiated

        init_level();
        init_styx(&styx);
        init_cursor(&cur);

        //main loop
        do {

            osSleep(speed * PAUSE_LENGTH);

            game_over = advance_styx(&styx);
            keypress = checkKey();
            event = advance_cursor(keypress, &cur);
            switch (event) {
            case SELF_INTERSECT:
                game_over = TRUE;
                break;
            case PATH_MADE:
                partition(border, path, region1, region2);
                if (inside(&styx.lines[0].p0, region1)) {
                    poly_t *tmp = border;
                    border = region1;
                    region1 = tmp;
                    size = fill_poly(region2, level);
                } else {
                    poly_t *tmp = border;
                    border = region2;
                    region2 = tmp;
                    size = fill_poly(region1, level);
                }
                score += size;
                level_score += size;
                path->n = -1;
                if (level_score > (BOARD_XMAX-BOARD_XMIN)*(BOARD_YMAX-BOARD_YMIN)/2) {
                  int i;
                  for (i=BOARD_YMIN+1; i<BOARD_YMAX; i++)
                    fill_line(i,BOARD_XMIN+1, BOARD_XMAX-1, level);
                  lcdGotoxy(0,110);
                  lcdPuts(" Level done ");
                  for (i=0; i<3; i++) {
                    setBuzzer(TRUE);
                    osSleep(1);
                    setBuzzer(FALSE);
                    osSleep(1);
                  }
                  while(checkKey() != KEY_CENTER);
                  level = (level + 1) % MAX_LEVEL;
                  level_score = 0;
                  speed = (speed - 2) % 14;
                  init_level();
                  init_styx(&styx);
                  init_cursor(&cur);
                }
                showScore();
                break;
            default:
                break;
            }

        } while (!game_over && keypress != KEY_CENTER);

        //game over message
        if (score > high_score)
            high_score = score;
        showScore();

        {
            tMenu menu;

            menu.xPos = 10;
            menu.yPos = 40;
            menu.xLen = 6 + (12 * 8);
            menu.yLen = 4 * 14;
            menu.noOfChoices = 2;
            menu.initialChoice = 0;
            menu.pHeaderText = "Game over!";
            menu.headerTextXpos = 20;
            menu.pChoice[0] = "Restart game";
            menu.pChoice[1] = "End game";
            menu.bgColor = 0;
            menu.borderColor = 0x6d;
            menu.headerColor = 0;
            menu.choicesColor = 0xfd;
            menu.selectedColor = 0xe0;

            done = drawMenu(menu);
        }

    } while (!done);
}


/*****************************************************************************
 *
 * Description:
 *    
 *
 ****************************************************************************/
void init_styx(styx_t * styx)
{
    line_t init;
    init.p0.x = init.p1.x = (BOARD_XMIN + BOARD_XMAX) / 2;
    init.p0.y = init.p1.y = (BOARD_YMIN + BOARD_YMAX) / 2;
    
    do {
        styx->delta.p0.x = (rand() % 8) - 4;
        styx->delta.p0.y = (rand() % 8) - 4;
    } while (styx->delta.p0.x==0 && styx->delta.p1.y==0);
    do {
        styx->delta.p1.x = (rand() % 8) - 4;
        styx->delta.p1.y = (rand() % 8) - 4;
    } while (styx->delta.p1.x==0 && styx->delta.p1.y==0);
    
    styx->color = 0;
    for (styx->current = 0; styx->current < STYX_LENGTH; styx->current++) {
        styx->lines[styx->current].p0.x = init.p0.x += styx->delta.p0.x;
        styx->lines[styx->current].p0.y = init.p0.y += styx->delta.p0.y;
        styx->lines[styx->current].p1.x = init.p1.x += styx->delta.p1.x;
        styx->lines[styx->current].p1.y = init.p1.y += styx->delta.p1.y;
        ++(styx->color);
        styx->color %= STYX_COLOR_CYCLE;
        draw_line(&styx->lines[styx->current], styx->color + STYX_COLOR_BASE);
    }
    styx->current = STYX_LENGTH - 1;
}

/*****************************************************************************
 *
 * Description:
 *    
 *
 ****************************************************************************/
int advance_styx(styx_t * styx)
{
    line_t *old, *new;
    int p0, p1, s;

    old = &styx->lines[styx->current];
    styx->current = (styx->current + 1) % STYX_LENGTH;
    new = &styx->lines[styx->current];

    draw_line(new, BG_COLOR);   /* Erase last line */

    new->p0.x = old->p0.x + styx->delta.p0.x;
    new->p0.y = old->p0.y + styx->delta.p0.y;
    new->p1.x = old->p1.x + styx->delta.p1.x;
    new->p1.y = old->p1.y + styx->delta.p1.y;

    p0 = inside(&new->p0, border);
    p1 = inside(&new->p1, border);

    if (!p0) {
        s = check_line(new, border);
        /* Bounce enpoint p0 */
        if (border->p[s].x != border->p[s + 1].x) {
            /* Horizontal segment */
            styx->delta.p0.y *= -1;
            new->p0.y = old->p0.y;
        } else {
            /* Vertical segment  */
            styx->delta.p0.x *= -1;
            new->p0.x = old->p0.x;
        }
    }
    if (!p1) {
        s = check_line(new, border);
        /* Bounce enpoint p1 */
        if (border->p[s].x != border->p[s + 1].x) {
            /* Horizontal segment */
            styx->delta.p1.y *= -1;
            new->p1.y = old->p1.y;
        } else {
            /* Vertical segment  */
            styx->delta.p1.x *= -1;
            new->p1.x = old->p1.x;
        }
    }
    s = check_line(new, border);
    if (p0 && p1 && -1 != s) {
        /* Bounce whole line */
        styx->delta.p0.x *= -1;
        styx->delta.p0.y *= -1;
        styx->delta.p1.x *= -1;
        styx->delta.p1.y *= -1;
        new->p0.x = old->p0.x + styx->delta.p0.x;
        new->p0.y = old->p0.y + styx->delta.p0.y;
        new->p1.x = old->p1.x + styx->delta.p1.x;
        new->p1.y = old->p1.y + styx->delta.p1.y;
    }

    /* TODO Fix
       {
       int dx = new->p1.x - new->p0.x;
       int dy = new->p1.y - new->p0.y;
       if (STYX_MAX_STREACH < dx*dx+dy*dy) {
       if (rand() & 0x08) {
       styx->delta.p0.x = styx->delta.p1.y;
       styx->delta.p0.y = styx->delta.p1.x;
       } else {
       styx->delta.p1.x = styx->delta.p0.y;
       styx->delta.p1.y = styx->delta.p0.x;
       }
       }
       }
     */

    ++(styx->color);
    styx->color %= STYX_COLOR_CYCLE;

    draw_line(new, styx->color + STYX_COLOR_BASE);

    if (-1 != check_line(new, path))
        return TRUE;

    return FALSE;
}


/*****************************************************************************
 *
 * Description:
 *    
 *
 ****************************************************************************/
void init_cursor(cursor_t * cur)
{
    cur->p.x = border->p[0].x;
    cur->p.y = border->p[0].y;
    cur->direction = KEY_NOTHING;
    cur->state = 0;
}


/*****************************************************************************
 *
 * Description:
 *    
 *
 ****************************************************************************/
int advance_cursor(const uint8_t key, cursor_t * cur)
{
    uint8_t direction;
    point_t nxt;
    int res = 0;

    // Update cur;
    nxt.x = cur->p.x;
    nxt.y = cur->p.y;
    switch (key) {
    case KEY_UP:
        nxt.y -= CURSOR_STEP;
        direction = key;
        break;
    case KEY_RIGHT:
        nxt.x += CURSOR_STEP;
        direction = key;
        break;
    case KEY_DOWN:
        nxt.y += CURSOR_STEP;
        direction = key;
        break;
    case KEY_LEFT:
        nxt.x -= CURSOR_STEP;
        direction = key;
        break;
    case KEY_NOTHING:
        return 0;
    default:
        direction = key;
        break;
    }

    if (-1 == path->n) {        /* Crusing the boarders */
        if (-1 != check_point(&nxt, border)) {  /* Still crusing move */
            /* Valid move */
        } else if (inside(&nxt, border)) {      /* We've started a new path */
            /* Valid move */
            path->n = 1;
            path->p[0].x = cur->p.x;
            path->p[0].y = cur->p.y;
            path->p[1].x = nxt.x;
            path->p[1].y = nxt.y;
            cur->direction = direction;
        } else {
            /* Invalid move */
            return 0;
        }
    } else {                    /* Carving a new path */
        if (KEY_NOTHING != key && -1 != check_point(&nxt, path)) {
            path->p[path->n].x = nxt.x;
            path->p[path->n].y = nxt.y;
            return SELF_INTERSECT;      /* Cannot cross own path */
        } else if (-1 != check_point(&nxt, border)) {
            /* We're safe on the borders again */
            /* Valid move */
            path->p[path->n].x = nxt.x;
            path->p[path->n].y = nxt.y;
            cur->direction = direction = KEY_NOTHING;
            res = PATH_MADE;
        } else {
            /* Valid move in free space */
            if (KEY_NOTHING != direction && direction != cur->direction) {
                path->n++;
                cur->direction = direction;
            }
            path->p[path->n].x = nxt.x;
            path->p[path->n].y = nxt.y;
        }
    }

    /* Draw new cursor */
    draw_ortho(&cur->p, &nxt, LINE_COLOR);

    cur->p.x = nxt.x;
    cur->p.y = nxt.y;

    return res;
}


/*****************************************************************************
 *
 * Description:
 *    Initialize one level of the game. Draw game board.
 *
 ****************************************************************************/
void init_level(void)
{
    int i;
    //clear screen
    lcdColor(BG_COLOR, LINE_COLOR);
    lcdClrscr();

    border = &regions[0];
    path = &regions[1];
    region1 = &regions[2];
    region2 = &regions[3];

    border->n = 4;
    border->p[0].x = BOARD_XMIN;
    border->p[0].y = BOARD_YMIN;
    border->p[1].x = BOARD_XMIN;
    border->p[1].y = BOARD_YMAX;
    border->p[2].x = BOARD_XMAX;
    border->p[2].y = BOARD_YMAX;
    border->p[3].x = BOARD_XMAX;
    border->p[3].y = BOARD_YMIN;
    border->p[4].x = BOARD_XMIN;
    border->p[4].y = BOARD_YMIN;

    for (i = 0; i < 4; i++)
        draw_ortho(&border->p[i], &border->p[i + 1], LINE_COLOR);

    path->n = -1;
    region1->n = -1;
    region2->n = -1;

    showScore();
}


/*****************************************************************************
 *
 * Description:
 *    Draw current score
 *
 ****************************************************************************/
void showScore(void)
{
    unsigned char buffer[20];
    lcdGotoxy(0,0);
    snprintf(buffer, 17, " Score  %06d", score);
    lcdPuts(buffer);
}

/*****************************************************************************
 *
 * Description:
 *    Clips enpoints to be within board
 *    Not correct, just moves enpoints orthogonaly to be within board
 *
 ****************************************************************************/
void clip_line(line_t * l)
{
    if (l->p0.x < BOARD_XMIN)
        l->p0.x = BOARD_XMIN + 1;
    if (l->p0.x > BOARD_XMAX)
        l->p0.x = BOARD_XMAX - 1;
    if (l->p0.y < BOARD_YMIN)
        l->p0.y = BOARD_YMIN + 1;
    if (l->p0.y > BOARD_YMAX)
        l->p0.y = BOARD_YMAX - 1;
    if (l->p1.x < BOARD_XMIN)
        l->p1.x = BOARD_XMIN + 1;
    if (l->p1.x > BOARD_XMAX)
        l->p1.x = BOARD_XMAX - 1;
    if (l->p1.y < BOARD_YMIN)
        l->p1.y = BOARD_YMIN + 1;
    if (l->p1.y > BOARD_YMAX)
        l->p1.y = BOARD_YMAX - 1;
}

void draw_ortho(const point_t * p0, const point_t * p1, const uint8_t color)
{
    /* Assumes either p0.x==p1.x OR p0.y==p1.y */

    uint8_t x, xmin, xmax;
    uint8_t y, ymin, ymax;

    xmin = (uint8_t) (p0->x < p1->x) ? p0->x : p1->x;
    xmax = (uint8_t) (p0->x > p1->x) ? p0->x : p1->x;
    ymin = (uint8_t) (p0->y < p1->y) ? p0->y : p1->y;
    ymax = (uint8_t) (p0->y > p1->y) ? p0->y : p1->y;

    selectLCD(TRUE);
    lcdWrcmd(LCD_CMD_CASET);    //set X
    lcdWrdata(xmin);
    lcdWrdata(xmax);

    lcdWrcmd(LCD_CMD_PASET);    //set Y
    lcdWrdata(ymin);
    lcdWrdata(ymax);

    lcdWrcmd(LCD_CMD_RAMWR);    //write memory
    for (x = xmin, y = ymin; x <= xmax || y <= ymax; x++, y++)
        lcdWrdata(color);

    selectLCD(FALSE);
}

/*****************************************************************************
 *
 * Description:
 *    Bresenham's line algorithm working on framebuffer
 *
 ****************************************************************************/
void draw_line(const line_t * l, const uint8_t color)
{

    int x, deltax, xinc1, xinc2;
    int y, deltay, yinc1, yinc2;
    int num, numadd, den;

    deltax = abs(l->p1.x - l->p0.x);
    deltay = abs(l->p1.y - l->p0.y);

    if (l->p1.x >= l->p0.x) {
        xinc1 = 1;
        xinc2 = 1;
    } else {
        xinc1 = -1;
        xinc2 = -1;
    }

    if (l->p1.y >= l->p0.y) {
        yinc1 = 1;
        yinc2 = 1;
    } else {
        yinc1 = -1;
        yinc2 = -1;
    }

    if (deltax >= deltay) {
        xinc1 = 0;
        yinc2 = 0;
        den = deltax;
        num = deltax / 2;
        numadd = deltay;
    } else {
        xinc2 = 0;
        yinc1 = 0;
        den = deltay;
        num = deltay / 2;
        numadd = deltax;
    }

    for (x = l->p0.x, y = l->p0.y; x != l->p1.x && y != l->p1.y; x += xinc2, y += yinc2) {
        put_pixel(x, y, color);
        num += numadd;
        if (num >= den) {
            num -= den;
            x += xinc1;
            y += yinc1;
        }
    }
}


/*****************************************************************************
 *
 * Description:
 *    Returns TRUE iff the point lies in the interior of the closed polygon
 *    points on the polygon are considered as outside.
 *
 ****************************************************************************/
int inside(const point_t * pt, const poly_t * py)
{
    int i, res = FALSE;
    for (i = 0; i < py->n; i++)
        if (py->p[i].x >= pt->x && (((py->p[i].y <= pt->y) && (pt->y < py->p[i + 1].y))
                                    || ((py->p[i].y > pt->y) && (pt->y >= py->p[i + 1].y)))) {
            res = !res;
        }
    return res && -1 == check_point(pt, py);
}


/*****************************************************************************
 *
 * Description:
 *    Returns the index of the segment the point coincides with or -1 if the 
 *    point isn't on any segment of the polygon.
 *
 ****************************************************************************/
int check_point(const point_t * pt, const poly_t * py)
{
    int i;
    for (i = 0; i < py->n; i++)
        if ((py->p[i].x == pt->x && pt->x == py->p[i + 1].x &&
             ((py->p[i].y <= pt->y && pt->y <= py->p[i + 1].y) ||
              (py->p[i].y >= pt->y && pt->y >= py->p[i + 1].y))) ||
            (py->p[i].y == pt->y && pt->y == py->p[i + 1].y &&
             ((py->p[i].x <= pt->x && pt->x <= py->p[i + 1].x)
              || (py->p[i].x >= pt->x && pt->x >= py->p[i + 1].x))))
            return i;
    return -1;
}


/*****************************************************************************
 *
 * Description:
 *    Returns the index of the first segment that the line crosses 
 *    or -1 if it crosses none.
 *
 * TODO Potential DIVIDE BY ZERO if zero-length line coincides with a segment
 *      Hopefully never hapens....
 *
 ****************************************************************************/
int check_line(const line_t * l, const poly_t * p)
{
    int x, y, i;
    for (i = 0; i < p->n; i++) {
        if (p->p[i].x != p->p[i + 1].x) {
            /* Horizontal segment */
            y = p->p[i].y;
            if ((l->p0.y <= y && y <= l->p1.y) || (l->p0.y >= y && y >= l->p1.y)) {
                /* Ignore round-off errors ... */
                x = l->p0.x + ((l->p1.x - l->p0.x) * (y - l->p0.y)) / (l->p1.y - l->p0.y);
                /* ... compensate by less strict interval test */
                if ((p->p[i].x <= x && x <= p->p[i + 1].x)
                    || (p->p[i].x >= x && x >= p->p[i + 1].x))
                    return i;
            }
        } else {
            /* Vertical segment */
            x = p->p[i].x;
            if ((l->p0.x <= x && x <= l->p1.x) || (l->p0.x >= x && x >= l->p1.x)) {
                y = l->p0.y + ((l->p1.y - l->p0.y) * (x - l->p0.x)) / (l->p1.x - l->p0.x);
                if ((p->p[i].y <= y && y <= p->p[i + 1].y)
                    || (p->p[i].y >= y && y >= p->p[i + 1].y))
                    return i;
            }
        }
    }
    return -1;
}


/*****************************************************************************
 *
 * Description:
 *    Partitions closed polygon b by open polygon p 
 *    into two closed polygons r1 and r2.
 *    Endpoints of p must coinside with b, or result is undefined.
 *
 ****************************************************************************/
void partition(const poly_t * b, const poly_t * p, poly_t * r1, poly_t * r2)
{
    int i;
    int start_seg, end_seg;
    const point_t *s, *e;
    r1->n = -1;
    r2->n = -1;
    s = &(p->p[0]);
    e = &(p->p[p->n]);
    start_seg = check_point(s, b);
    end_seg = check_point(e, b);
    if (start_seg < end_seg) {
        for (i = 0; i <= start_seg; i++) {
            r1->n++;
            r1->p[r1->n].x = b->p[i].x;
            r1->p[r1->n].y = b->p[i].y;
        }
        for (i = 0; i <= p->n; i++) {
            r1->n++;
            r1->p[r1->n].x = p->p[i].x;
            r1->p[r1->n].y = p->p[i].y;
        }
        for (i = end_seg + 1; i <= b->n; i++) {
            r1->n++;
            r1->p[r1->n].x = b->p[i].x;
            r1->p[r1->n].y = b->p[i].y;
        }
        r2->n = 0;
        r2->p[0].x = p->p[0].x;
        r2->p[0].y = p->p[0].y;
        for (i = start_seg + 1; i <= end_seg; i++) {
            r2->n++;
            r2->p[r2->n].x = b->p[i].x;
            r2->p[r2->n].y = b->p[i].y;
        }
        for (i = p->n; i >= 0; i--) {
            r2->n++;
            r2->p[r2->n].x = p->p[i].x;
            r2->p[r2->n].y = p->p[i].y;
        }
    } else if (start_seg > end_seg) {
        for (i = 0; i <= end_seg; i++) {
            r1->n++;
            r1->p[r1->n].x = b->p[i].x;
            r1->p[r1->n].y = b->p[i].y;
        }
        for (i = p->n; i >= 0; i--) {
            r1->n++;
            r1->p[r1->n].x = p->p[i].x;
            r1->p[r1->n].y = p->p[i].y;
        }
        for (i = start_seg + 1; i <= b->n; i++) {
            r1->n++;
            r1->p[r1->n].x = b->p[i].x;
            r1->p[r1->n].y = b->p[i].y;
        }
        r2->n = 0;
        r2->p[0].x = p->p[p->n].x;
        r2->p[0].y = p->p[p->n].y;
        for (i = end_seg + 1; i <= start_seg; i++) {
            r2->n++;
            r2->p[r2->n].x = b->p[i].x;
            r2->p[r2->n].y = b->p[i].y;
        }
        for (i = 0; i <= p->n; i++) {
            r2->n++;
            r2->p[r2->n].x = p->p[i].x;
            r2->p[r2->n].y = p->p[i].y;
        }
    } else {
        for (i = 0; i <= start_seg; i++) {
            r1->n++;
            r1->p[r1->n].x = b->p[i].x;
            r1->p[r1->n].y = b->p[i].y;
        }
        if (abs(s->x - b->p[start_seg].x + s->y - b->p[start_seg].y) <
            abs(e->x - b->p[start_seg].x + e->y - b->p[start_seg].y)) {
            for (i = 0; i <= p->n; i++) {
                r1->n++;
                r1->p[r1->n].x = p->p[i].x;
                r1->p[r1->n].y = p->p[i].y;
            }
            for (i = p->n; i >= 0; i--) {
                r2->n++;
                r2->p[r2->n].x = p->p[i].x;
                r2->p[r2->n].y = p->p[i].y;
            }
            r2->n++;
            r2->p[r2->n].x = p->p[p->n].x;
            r2->p[r2->n].y = p->p[p->n].y;
        } else {
            for (i = p->n; i >= 0; i--) {
                r1->n++;
                r1->p[r1->n].x = p->p[i].x;
                r1->p[r1->n].y = p->p[i].y;
            }
            r2->n++;
            r2->p[r2->n].x = p->p[p->n].x;
            r2->p[r2->n].y = p->p[p->n].y;
            for (i = 0; i <= p->n; i++) {
                r2->n++;
                r2->p[r2->n].x = p->p[i].x;
                r2->p[r2->n].y = p->p[i].y;
            }
        }
        for (i = end_seg + 1; i <= b->n; i++) {
            r1->n++;
            r1->p[r1->n].x = b->p[i].x;
            r1->p[r1->n].y = b->p[i].y;
        }
    }
}


/*****************************************************************************
 *
 * Description:
 *
 *    Fills the closed polygon p with pixels from image level
 *    Returns the number of pixels filled
 *
 ****************************************************************************/
int fill_poly(const poly_t * p, const int level)
{
    int nedges, i, j;
    int edges[POLY_LENGTH * 2]; /* You could probably argue for a smaller buffer, but ... */
    int x, y, size;
    size = 0;
    for (y = BOARD_YMIN; y <= BOARD_YMAX; y++) {
        nedges = 0;
        for (i = 0; i < p->n; i++) {
            /* if ((p->p[i].y <= y && y < p->p[i + 1].y) || (p->p[i].y > y && y >= p->p[i + 1].y)) { */
            if (p->p[i].y <= y && y < p->p[i + 1].y) {
                /* Current segment crosses this scanline. 
                   Do insertionsort into edgelist */
                x = p->p[i].x;
                for (j = nedges; j > 0 && x < edges[j - 1]; j--)
                    edges[j] = edges[j - 1];
                edges[j] = x;
                nedges++;
            } else if (p->p[i+1].y <= y && y < p->p[i].y) {
                x = p->p[i].x;
                for (j = nedges; j > 0 && x < edges[j - 1]; j--)
                    edges[j] = edges[j - 1];
                edges[j] = x;
                nedges++;
          }
        }
        /* Skip first and last as they are "outside" */
        for (i = 0; i < (nedges-1); i += 2)
            size += fill_line(y, edges[i] + 1, edges[i + 1] - 1, level);
    }
    /* Polygon fill is buggy, overwrites some horizontal segments.
       Redraw them and let the user have som bonus points for bugs */
    for (i=0; i < p->n; i++)
      draw_ortho(&p->p[i], &p->p[i+1], LINE_COLOR);
    
    return size;
}

/*****************************************************************************
 *
 * Description:
 *
 *    Fills the horizontal line from (x0,y) to (x1,y) inclusive with pixels
 *    from compressed picture pointed to by levelmap[level]
 *    Returns the number of pixels filled, ie. length of the line
 *
 ****************************************************************************/
int fill_line(const int y, const int x0, const int x1, const int level)
{
  static int l = -1, o = INT_MAX;
  static const unsigned char *p;
  int n,c,i=0;
  int ostart, ostop;
  unsigned char escapeChar;

  if (x1 >= x0) {
    escapeChar = levelmap[level][3];
    ostart = y*levelmap[level][0] + x0;
    ostop  = y*levelmap[level][0] + x1;
    if (level != l || ostart < o) {
      /* Reset pointer */
      l = level;
      p = levelmap[l] + 4;
      o = 0;
    }
    n=0;
    c=0;
    if (levelmap[l][2]) {
      while( o < ostart) {
        if (*p++ == escapeChar) {
          n = *p++;
          c = *p++;
          if ((o+n)>ostart) break;
          o += n;
        } else
          o++;
      }
    } else {
      p = levelmap[l] + 4 + ostart;
      o = ostart;
    }

    selectLCD(TRUE);

    lcdWrcmd(LCD_CMD_CASET);        //set X
    lcdWrdata((uint8_t) x0);
    lcdWrdata((uint8_t) x1);

    lcdWrcmd(LCD_CMD_PASET);        //set Y
    lcdWrdata((uint8_t) y);
    lcdWrdata((uint8_t) y);

    lcdWrcmd(LCD_CMD_RAMWR);        //write memory

    if (levelmap[l][2]) {
      /* Paint reminder of previous eventual Run-Length Encoded segment */
      if (o<ostart) {
        n -= (ostart - o);
        o = ostart;
        for (i=0; i<n && o<=ostop; i++, o++)
          lcdWrdata(c);
      }
      while (o <= ostop) {
        if ((c = *p++) == escapeChar) {
          n = *p++;
          c = *p++;
          for (i=0; i<n && o<=ostop; i++, o++)
            lcdWrdata(c);
        } else {
          lcdWrdata(c);
          o++;
        }
      }
      if (i<n) {
        /* We ended in a RLE segment, 
           rollback to reuse segment on next pass */
        p -= 3;
        o -= i;
      }
    } else {
      for (;o<=ostop; o++) {
        lcdWrdata(*p++);
      }
    }

    selectLCD(FALSE);
    
    return (x1 - x0 + 1);
  }
  return 0;
}


void put_cursor(const point_t * c)
{
    int x, y;
    selectLCD(TRUE);

    lcdWrcmd(LCD_CMD_CASET);    //set X
    lcdWrdata((uint8_t) (c->x - 1));
    lcdWrdata((uint8_t) (c->x + 1));

    lcdWrcmd(LCD_CMD_PASET);    //set Y
    lcdWrdata((uint8_t) (c->y - 1));
    lcdWrdata((uint8_t) (c->y + 1));

    lcdWrcmd(LCD_CMD_RAMWR);    //write memory
    for (y = c->y - 1; y <= c->y + 1; y++)
        for (x = c->x - 1; x <= c->x + 1; x++)
            lcdWrdata(LINE_COLOR);

    selectLCD(FALSE);
}


void put_pixel(const int x, const int y, const uint8_t color)
{
    selectLCD(TRUE);

    lcdWrcmd(LCD_CMD_CASET);    //set X
    lcdWrdata((uint8_t) x);
    lcdWrdata((uint8_t) x);

    lcdWrcmd(LCD_CMD_PASET);    //set Y
    lcdWrdata((uint8_t) y);
    lcdWrdata((uint8_t) y);

    lcdWrcmd(LCD_CMD_RAMWR);    //write memory
    lcdWrdata(color);

    selectLCD(FALSE);
}
